//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

/* ------------------------- */
/* --- Define                */
/* ------------------------- */

#pragma abs_address 0x7FC0    //@EEPROM_1 
 const WORD Initial50Zero = 0; 
 const WORD Initial33Zero = 0; 
#pragma end_abs_address

/* --- wait counter (32ms) */
static BYTE g_waitint;
static BYTE g_modeled;

/* --- LCD Dynamic drive */
static BYTE g_dynpos;
static BYTE g_segment[4];
static BYTE g_pilotled;
static BYTE g_point;

const BYTE d_7seghex[] = {
	0b11000000,
	0b11111001,
	0b10100100,
	0b10110000,
	0b10011001,
	0b10010010,
	0b10000010,
	0b11011000,
	0b10000000,
	0b10010000,
	0b10001000,
	0b10000011,
	0b11000110,
	0b10100001,
	0b10000110,
	0b10001110,
	0b11111111,
	0b01111111,
	0b11110000,
};

/* --- zero level sample */
static WORD g_zero50;
static WORD g_zero33;

/* --- mode */
static BYTE g_mode;


/* --------------------------------------- */
/* --- interrupt                           */
/* --------------------------------------- */
#pragma interrupt_handler LCDInt
void LCDInt(void)
{
	BYTE pos, seg;
	g_dynpos = (g_dynpos + 1) & 0x03;
	pos = (0b00010000 << g_dynpos) | 0b00000111;
	if (g_pilotled == 0) {
		pos |= 0b00001000;
	}
	if (g_point == (g_dynpos+1)) {
		seg = g_segment[3-g_dynpos] & 0b01111111;
	}
	else {
		seg = g_segment[3-g_dynpos];
	}
	PRT1DR = (pos & 0b00001111);
	PRT2DR = seg;
	PRT1DR = pos;
}


#pragma interrupt_handler TimerInt
void TimerInt(void)
{
	if (g_waitint > 0) {
		g_waitint--;
	}
	
	g_modeled++;
	if ((g_modeled & 0b00000100) && (g_modeled & 0b00000010) && (g_mode)) {
		g_pilotled = 0;
	}
	else {
		g_pilotled = 1;
	}
}

void Wait(BYTE count)
{
	g_waitint = count;
	while(g_waitint) {}
}

/* --------------------------- */
/* --- read A                  */
/* --------------------------- */
// 0=5.0V / 1=3.3V
void select_mux(BYTE in)
{
	if (in) {
		AMX_IN = 0b01010101;
	}
	else {
		AMX_IN = 0b00000000;
	}
}

WORD get_sample(void)
{
	WORD  sample;

	while(ADCINC_1_fIsDataAvailable() == 0) {};

	sample = ADCINC_1_wGetData();
	ADCINC_1_fClearFlag();
	
	return sample;
}

/* --------------------------- */
/* --- LED controll            */
/* --------------------------- */

void disp_hex16bit(WORD dat)
{
	BYTE b;
	b = dat & 0xff;
	g_segment[3] = d_7seghex[b & 0x0f];
	g_segment[2] = d_7seghex[(b >> 4) & 0x0f];
	b = (dat >> 8) & 0xff;
	g_segment[1] = d_7seghex[b & 0x0f];
	g_segment[0] = d_7seghex[(b >> 4) & 0x0f];
}

void to_50v(void)
{
	WORD t;

	select_mux(0);
	g_segment[0] = d_7seghex[16];
	g_segment[1] = d_7seghex[16];
	g_segment[2] = d_7seghex[16];
	g_segment[3] = d_7seghex[16];
	g_point = 0;
	Wait(3);
	// 1
	g_segment[3] = d_7seghex[12];
	Wait(3);
	// 2
	g_segment[2] = g_segment[3];
	g_segment[3] = d_7seghex[5];
	g_point = 1;
	Wait(3);
	// 3
	g_segment[1] = g_segment[2];
	g_segment[2] = g_segment[3];
	g_segment[3] = d_7seghex[0];
	g_point = 2;
	Wait(3);
	// 4
	g_segment[0] = g_segment[1];
	g_segment[1] = g_segment[2];
	g_segment[2] = g_segment[3];
	g_segment[3] = d_7seghex[18];
	g_point = 3;
	//
	t = get_sample();
	Wait(15);
}

void to_33v(void)
{
	WORD t;

	select_mux(1);
	g_segment[0] = d_7seghex[16];
	g_segment[1] = d_7seghex[16];
	g_segment[2] = d_7seghex[16];
	g_segment[3] = d_7seghex[16];
	g_point = 0;
	Wait(3);
	// 1
	g_segment[3] = d_7seghex[12];
	Wait(3);
	// 2
	g_segment[2] = g_segment[3];
	g_segment[3] = d_7seghex[3];
	g_point = 1;
	Wait(3);
	// 3
	g_segment[1] = g_segment[2];
	g_segment[2] = g_segment[3];
	g_segment[3] = d_7seghex[3];
	g_point = 2;
	Wait(3);
	// 4
	g_segment[0] = g_segment[1];
	g_segment[1] = g_segment[2];
	g_segment[2] = g_segment[3];
	g_segment[3] = d_7seghex[18];
	g_point = 3;
	//
	t = get_sample();
	Wait(15);
}

void disp_ampere(WORD adc)
{
	DWORD ap;
	BYTE k[4];

	g_point = 2;
	k[3] = d_7seghex[16];
	k[2] = d_7seghex[16];
	k[1] = d_7seghex[16];
	k[0] = d_7seghex[16];
	//
	ap = (DWORD)adc;
	ap *= 7813;  // max 781.3 mA
	ap /= 0x3fff;
	//
	if (ap > 999) {
		k[0] = d_7seghex[(ap/1000)];
		k[1] = d_7seghex[0];
		k[2] = d_7seghex[0];
		k[3] = d_7seghex[0];
		ap = ap % 1000;
	}
	if (ap > 99) {
		k[1] = d_7seghex[(ap/100)];
		k[2] = d_7seghex[0];
		k[3] = d_7seghex[0];
		ap = ap % 100;
	}
	k[2] = d_7seghex[(ap/10)];
	k[3] = d_7seghex[(ap%10)];
	g_segment[0] = k[0];
	g_segment[1] = k[1];
	g_segment[2] = k[2];
	g_segment[3] = k[3];
}


/* --------------------------- */
/* --- access zero level       */
/* --------------------------- */

void read_zero_level(void)
{
	BYTE buf[4];
	E2PROM_1_E2Read(0, buf, 4);
	g_zero50 = buf[0] + (buf[1] << 8);
	g_zero33 = buf[2] + (buf[3] << 8);
}

void write_zero_level(WORD level50, WORD level33)
{
	BYTE buf[4];
	WORD tmp;

	tmp = level50 >> 8;
	buf[0] = level50 & 0xff;
	buf[1] = tmp & 0xff;
	tmp = level33 >> 8;
	buf[2] = level33 & 0xff;
	buf[3] = tmp & 0xff;
	/* write to E2PROM */
	E2PROM_1_bE2Write(0, buf, 4, 25);
}

void sample_zero_level(void)
{
	BYTE loop;
	WORD sample;
	g_segment[0] = d_7seghex[16];
	g_segment[1] = d_7seghex[16];
	g_segment[2] = d_7seghex[16];
	g_segment[3] = d_7seghex[16];
	g_point = 0;
	// - 5.0V
	select_mux(0);
	sample = get_sample();
	for(loop=0; loop<5; loop++) {
		Wait(1);
		sample = (sample + get_sample()) >> 1;
	}
	g_segment[3] = d_7seghex[17];
	for(loop=0; loop<5; loop++) {
		Wait(1);
		sample = (sample + get_sample()) >> 1;
	}
	g_segment[2] = d_7seghex[17];
	if (sample > 0x20) {
		sample -= 0x20;
	}
	g_zero50 = sample;
	// - 3.3V
	select_mux(1);
	sample = get_sample();
	for(loop=0; loop<5; loop++) {
		Wait(1);
		sample = (sample + get_sample()) >> 1;
	}
	g_segment[1] = d_7seghex[17];
	for(loop=0; loop<5; loop++) {
		Wait(1);
		sample = (sample + get_sample()) >> 1;
	}
	g_segment[0] = d_7seghex[17];
	if (sample > 0x20) {
		sample -= 0x20;
	}
	g_zero33 = sample;
	Wait(5);
	//
	select_mux(0);
	write_zero_level(g_zero50, g_zero33);
}

/* --------------------------- */
/* --- main routine            */
/* --------------------------- */
void main(void)
{
	BYTE count, b;
	WORD dat;

	g_waitint = 0;
	g_dynpos = 0;
	g_pilotled = 1;
	g_point = 0;
	g_mode = 0;
	g_modeled = 0;
	
	/* --- module initialize */
	Counter16_1_EnableInt();
    Counter16_1_Start();
	Counter8_1_EnableInt();
    Counter8_1_Start();
	INSAMP_1_Start(2); 
	PGA_1_Start(2);
	ADCINC_1_Start(2);
	E2PROM_1_Start();

	M8C_EnableGInt;

	count = 0;
	ADCINC_1_GetSamples(0);
	ADCINC_1_fClearFlag();

	/* --- read zero from E2PROM */
	read_zero_level();
	PRT1DR = 0b00001111;
	Wait(1);
	b = PRT1DR & 0b00000100;
	if (b == 0) {
		sample_zero_level();
	}
	to_50v();

	while(1) {
		Wait(1);
		dat = get_sample();
		if (dat < g_zero50) {
			dat = 0;
		}
		else {
			dat -= g_zero50;
		}
		disp_ampere(dat);

		count += 1;
		if ((PRT1DR & 0b00000100) == 0) {
			if (g_mode) {
				g_mode = 0;
				to_50v();
			}
			else {
				g_mode = 1;
				to_33v();
			}
		}
	}
	
}
